#  include <boost/units/systems/si.hpp>
using namespace boost::units;

#  include <blitz/array.h>
using namespace blitz;

typedef double real_t;
typedef quantity<si::dimensionless, real_t> quant;

/// gnuplot-digestible output for Blitz
ostream& operator<<(ostream& os, const Array<quant, 1>& x)
{
  for (int i=x.lbound(0); i<=x.ubound(0); ++i) os << real_t(x(i)) << endl;
  return os; 
}

// advection operator
class adv_upstream
{
  public: void op(Array<quant, 1> *psi[], const Range &i, 
    const int n, const quantity<si::dimensionless, real_t> &C)
  {
    (*psi[n+1])(i) -= C * ( 
      C > 0.  
        ? (*psi[n])(i) - (*psi[n])(i-1) 
        : (*psi[n])(i+1) - (*psi[n])(i)
    );  
  }
};

// main
int main()
{
  // parameters (FIXME: should be taken from command-line options)
  int nx = 32, nt = 64;
  quantity<si::length, real_t> dx = 1 * si::metres;
  quantity<si::time, real_t> dt = 1 * si::seconds;
  quantity<si::velocity, real_t> u = -.1 * si::metres / si::seconds;
  quantity<si::dimensionless, real_t> C = u * dt / dx;

  // memory allocation (FIXME: no deallocation yet!)
  Array<quant, 1> *psi[2];
  for (int n=0; n < 2; ++n) psi[n] = new Array<quant, 1>(Range(-1,nx));
  adv_upstream *adv = new adv_upstream();

  // initial condition 
  (*psi[0])(Range(nx/4, nx/2)) = 1;

  // integration loop
  for (int t = 0; t < nt; ++t)
  {
    // output
    cout << *psi[0] << endl << endl;

    // copying data from psi[n] -> psi[n+1]
    *psi[1] = *psi[0];

    // periodic boundary
    (*psi[1])(Range(-1)) = (*psi[0])(Range(nx-1));
    (*psi[1])(Range(nx)) = (*psi[0])(Range(0));

    // advection operator
    adv->op(psi, Range(0, nx-1), 0, C);

    // swapping arrays
    cycleArrays(*psi[0], *psi[1]);
  }
}
